#include <mutek/startup.h>
#include <mutek/thread.h>
#include <mutek/printk.h>
#include <stdio.h>
#include <device/class/char.h>
#include <device/class/timer.h>
#include <device/class/pwm.h>

#define BACKLIGHT 12

static CONTEXT_ENTRY(lcd_hello)
{
    error_t err;
    struct device_char_s lcd;
    struct device_timer_s timer;
    struct device_pwm_s pwm;
    dev_timer_delay_t frame;
    struct dev_pwm_config_s pwm_config;

    pwm_config.freq.num = 16000;
    pwm_config.freq.denom = 1;
    pwm_config.duty.num = 1;
    pwm_config.duty.denom = 16;
    pwm_config.pol = DEV_PWM_POL_HIGH;

    err = device_get_accessor_by_path(&lcd.base, NULL, "/lcd*", DRIVER_CLASS_CHAR);
    assert(!err);

    err = device_get_accessor_by_path(&pwm.base, NULL, "/pwm*", DRIVER_CLASS_PWM);
    assert(!err);

    err = device_get_accessor_by_path(&timer.base, NULL, "/rtc* /timer*", DRIVER_CLASS_TIMER);
    assert(!err);

    device_start(&pwm.base);
    dev_pwm_wait_config(&pwm, &pwm_config,
                        DEV_PWM_MASK_DUTY|DEV_PWM_MASK_FREQ|DEV_PWM_MASK_POL);
    dev_timer_init_sec(&timer, &frame, 0, 1, 10);

    for (;;) {
        char buffer[32];

        for (uint8_t l = 0; l < 2; ++l) {
            for (uint8_t cc = 0; cc < 13; ++cc) {
                uint8_t c = l ? 12 - cc : cc;
                dev_timer_wait_delay(&timer, frame, 0);

                pwm_config.duty.num = c + 4;
                dev_pwm_wait_config(&pwm, &pwm_config, DEV_PWM_MASK_DUTY);

                size_t n = snprintf(buffer, 32, "\x1b[2J\x1b[%d;%dH%d:%d",
                         l, c, l, c);
                dev_char_wait_op(&lcd, DEV_CHAR_WRITE_FLUSH, (void *)buffer, n);
            }
        }
    }
}

void app_start()
{
  struct thread_attr_s attr = {
    .stack_size = 1024,
  };

  thread_create(lcd_hello, 0, &attr);
}
